TK_SYS_ACK = 0xF0100001
TK_SYS_NAK = 0xF0100002
TK_SYS_BAD_REQUEST = 0xF0100003
TK_SYS_APPLICATION_ERROR = 0xF0100004
TK_SYS_QUEUE_FULL = 0xF0100005
TK_SYS_HEALTH_CHECK = 0xF0100006

TK_HV_FIND_SERVICE = 0x00200005
TK_HV_START_COMP = 0x00200007
TK_HV_SERVICEONLINE = 0x00200008
TK_HV_SERVICEOFFLINE = 0x00200009

TK_CMP_TERM_NOW = 0xF0300001
TK_CMP_GETSERVICES = 0xF0300002
TK_CMP_GETDEPENDS = 0xF0300003
TK_CMP_REGSERVICE = 0xF0300004
TK_CMP_START = 0xF0300005
TK_CMP_RELOAD = 0xF0300007
TK_CMP_RECREATE = 0xF0300008
TK_CMP_HEALTH_CHECK = 0xF0300009

TK_EVT_EVENT = 0xF0400001
TK_EVT_SUBSCRIBE = 0xF0400002
TK_EVT_UNSUBSCRIBE = 0xF0400003
