from typing import Optional


class MBService:
    def __init__(
        self,
        address: str,
        port: int,
        name: Optional[str] = None,
    ):

        self.address = address
        self.port = port
        self.name = name

    def __str__(self):
        return f"{self.address}:{self.port} - {self.name}"
